import logging


RACKET_COLOR = (200, 0, 0)

BALL_COLOR = (173, 255, 47)

BOARD_COLOR = (233, 150, 122)

BOARD_SIZE = (700, 700)

RACKET_LENGTH = 50

RACKET_HEIGHT = 10

BALL_SPEED = 10

RACKET_SPEED = 5

WS_HEARTBEAT_TIMEOUT = 10

LOGGING_LEVEL = logging.DEBUG

SERVER_HOST = "0.0.0.0"

SERVER_PORT = 8888
